/*
	File:		FrazRegistry.h

	Contains:	Fragmalyzer AppleEvent Registry constants

	Written by:	Dan Wright

	Copyright:	 1998-2000 Dan Wright, All rights reserved

	Version:	1.5

	Change History (most recent first):

         <1>    10/17/00    DW      
*/


#ifndef __FRAZREGISTRY__

#define __FRAZREGISTRY__

#include <ASRegistry.h>
#include <AERegistry.h>
#include <FinderRegistry.h>

/*

	The Fragmalyzer Suite

*/

enum {

	kAEFragmalyzerSuite		= FOUR_CHAR_CODE('FraZ'),

	kAEFragmalyzerPluginSuite	= FOUR_CHAR_CODE('Fzec')

	};



/*

	The Fragmalyzer Events

*/

enum {

	kAEOpenContext		= FOUR_CHAR_CODE('octx'),

	kAEFindMinimalSet		= FOUR_CHAR_CODE('fmin'),

	kAEFindCompleteSet		= FOUR_CHAR_CODE('fall'),

	kAEFindLibrary			= FOUR_CHAR_CODE('fone')

	};

	

/*

	Events for standard plug-ins

*/

enum	{

	/* Members of Fragmalyzer Plug-in Suite */

	kAEMakeStub			= FOUR_CHAR_CODE('mstb'),

	kAEBrowseCode		= FOUR_CHAR_CODE('Brow'),

	kAEBrowseHex			= FOUR_CHAR_CODE('Hexa')

	};

	



/*

	Classes

*/

enum {

	cBrowser				= FOUR_CHAR_CODE('Brow'),
	
	cFragment				= FOUR_CHAR_CODE('Frag'),

#ifndef __ASREGISTRY__
	cSymbol				= FOUR_CHAR_CODE('symb'),
#endif

	cImportLibrary			= FOUR_CHAR_CODE('iLib'),

	cLocator				= FOUR_CHAR_CODE('cflc'),

	/* Introduced in version 1.5 */
	
	cFragmentSection		= FOUR_CHAR_CODE('zSct'),
	
	cFragmentSegment		= FOUR_CHAR_CODE('zSeg')	/* not currently supported for PEF */
	};


/*

	Types

*/

enum {
	typeBrowser			= cBrowser,
	
	typeFragment			= cFragment,
	
	typeLocator			= cLocator,
	
	typeFragmentSection		= cFragmentSection
	};

/*

	Properties of class cApplication

*/

enum	{

	pRegistered			= FOUR_CHAR_CODE('Zreg'),	/* boolean: is this copy registered? */

	pCompany				= FOUR_CHAR_CODE('Zomp'),	/* name of registered company */

#ifndef __FINDERREGISTRY__
	pOwner				= FOUR_CHAR_CODE('sown'),	/* name of registered user */
#endif
	
	pWebBrowser			= FOUR_CHAR_CODE('Zweb'),	/* preferred web browser */

	pTCPWillDial			= FOUR_CHAR_CODE('Zial')	/* will opening TCP/IP require the modem to dial? */

	};



/*

	Properties of class cWindow

*/

enum	{

	pReadOnly				= FOUR_CHAR_CODE('Zroa'),

	pContext				= FOUR_CHAR_CODE('Zctx')	/* CfragContextID/process serial number/process name */

	};



/*

	Properties of class cFragment

*/

enum	{

	pConnectionID			= FOUR_CHAR_CODE('Zcid'),	/* ...if a context browser */

	pContainerFile_OBSOLETE	= FOUR_CHAR_CODE('Zfss'),	/* use pLocator instead */

	pAppStackSize			= FOUR_CHAR_CODE('Zass'),	/* long: applications only */

	pAppLibFolder			= FOUR_CHAR_CODE('Zalf'),	/* alias to folder */

	pUsage				= FOUR_CHAR_CODE('Zuse'),	/* enumerated */

	pBuildDate				= FOUR_CHAR_CODE('Zbld'),	/* typeLongDateTime */

	pUpdateLevel			= FOUR_CHAR_CODE('Zupl'),	/* integer, 0 indicates base fragment */

	pArchitecture			= FOUR_CHAR_CODE('Zarc'),	/* enumerated */

	pSharingKind			= FOUR_CHAR_CODE('Zshk'),	/* (refers to sharing-kind of DATA section) */

#ifndef __AEREGISTRY__
	pVersion				= FOUR_CHAR_CODE('vers'),	/* numeric */
#endif
	
	pLocator				= cLocator,				/* location of fragment */

#ifndef __ASREGISTRY__
	pProperties			= FOUR_CHAR_CODE('pALL'),
#endif 

	/* Introduced in version 1.5 */
	
	pBinaryFormat			= FOUR_CHAR_CODE('BinF')	/* binary format */
	
	};



/*

	Properties of class cImportLibrary, cSymbol

*/

enum	{

	pWeakLink				= FOUR_CHAR_CODE('Zlnk')

	/* cImportLibrary also supports pName, pVersion */

	/* cSymbol also supports pName */

	};

	
/*

	Properties of class cFragmentSection

*/

enum	{
	
	pSectionKind			= pKind
	
	};
	
	
/*

	Enumerations

*/

enum	{
	/*   Architectures   */
	
	enumPowerPCArch		= FOUR_CHAR_CODE('pwpc'),

	enumMotorola68kArch	= FOUR_CHAR_CODE('m68k'),

	

	
	/*   Usages   */
	
	enumImportLibraryUsage	= FOUR_CHAR_CODE('Zus0'),

	enumApplicationUsage	= FOUR_CHAR_CODE('Zus1'),

	enumDropinAdditionUsage	= FOUR_CHAR_CODE('Zus2'),

	enumStubLibraryUsage	= FOUR_CHAR_CODE('Zus3'),

	enumWeakStubLibraryUsage = FOUR_CHAR_CODE('Zus4'),

	enumUnknownUsage		= FOUR_CHAR_CODE('Zus5'),

	

	
	/*   Locator kinds   */
	
	enumDataForkLocator	= FOUR_CHAR_CODE('dfrk'),

	enumResourceLocator	= FOUR_CHAR_CODE('rsrc'),

	enumMemoryLocator		= FOUR_CHAR_CODE('memo'),


	
	/*   Binary Formats   */
	
	enumPEFBinary			= FOUR_CHAR_CODE('peff'),
	
	enumXCOFFBinary		= FOUR_CHAR_CODE('xcof'),
	
	enumMachOBinary		= FOUR_CHAR_CODE('Mach')
	
	};





/*

	Keywords defined by the Fragmalyzer

*/

enum	{

	keyOffset				= FOUR_CHAR_CODE('Zoff'),

	keyResourceType		= FOUR_CHAR_CODE('rtyp'),

	keyResourceID			= FOUR_CHAR_CODE('rid '),

	keyAddress			= FOUR_CHAR_CODE('addr'),

	keyKind				= pKind,

	keyContainerFile		= FOUR_CHAR_CODE('Zfss'),	/* typeAlias */

	keyLength				= pLength

	};




#endif /* __FRAZREGISTRY__ */